﻿ConditionBlock = AbstractBlock.extend({

    NAME: "ConditionBlock",

    init: function (attr) {
        this._super($.extend({ resizeable: false, bgColor: "#F4F9FF", color: "#5A99FA", stroke: 0.5, radius: 3 }, attr));

        this.setWidth(290);
        this.setHeight(58);

        if (attr && ("id" in attr)) {
            this.setId(attr.id);
        }

        this.setUserData({});

        this.installEditPolicy(new CustomAntSelectionFeedbackPolicy());

        this.hexagonLocator = new TopLocatorWithMargin({ margin: -12 });
        this.falseLabelLocator = new draw2d.layout.locator.LeftLocator({ margin: -40 });
        this.trueLabelLocator = new draw2d.layout.locator.RightLocator({ margin: -40 });

        this.add(this.getHexagonFigure(), this.hexagonLocator);
        this.add(this.getTextLabel("false"), this.falseLabelLocator);
        this.add(this.getTextLabel("true"), this.trueLabelLocator);

        let blockNameTextLabel = this.getTextLabel(this.userData.blockName);
        blockNameTextLabel.userData = { id: "conditionName" };
        blockNameTextLabel.on("dblclick", () => { showModal(this); ConditionBlockContentHandler.initializeConditionModal(); });

        this.add(blockNameTextLabel, new draw2d.layout.locator.CenterLocator());

        this.on("dblclick", () => { showModal(this); ConditionBlockContentHandler.initializeConditionModal(); });
    },

    getPersistentAttributes: function () {
        var memento = this._super();

        let originalPorts = this.getPorts();

        for (let port of memento.ports) {
            let targetOriginalPort = originalPorts.find(f => f.id === port.id);
            let originalLocator = targetOriginalPort.locator;

            let locatorKeys = Object.keys(originalLocator)
                .filter(key => locatorAttributeNames.includes(key));

            for (let k of locatorKeys)
                port.locatorAttr[k] = originalLocator[k];
        }

        memento.childFigureLocators = {
            "Hexagon": this.hexagonLocator,
            "Label_false": this.falseLabelLocator,
            "Label_true": this.trueLabelLocator
        };

        return memento;
    },

    getHexagonFigure: function () {
        let hexagonFigure = new draw2d.shape.basic.Image({
            path: '/Images/BPM/BlueHexagon.svg',
            width: 118,
            height: 22,
            minWidth: 118,
            minHeight: 22,
            boundingBox: new draw2d.geo.Rectangle(0, 0, 118, 22),
            userData: {
                name: "Hexagon"
            }
        });

        let ifTextLabel = new draw2d.shape.basic.Label({
            text: localizedIfString,
            stroke: 0,
            radius: 0,
            bgColor: null,
            fontColor: "#fff",
            resizeable: true
        });

        hexagonFigure.add(ifTextLabel, new draw2d.layout.locator.CenterLocator());

        return hexagonFigure;
    },

    getTextLabel: function (labelText) {
        return new draw2d.shape.basic.Label({
            text: labelText,
            stroke: 0,
            radius: 0,
            bgColor: null,
            fontColor: "#000",
            resizeable: true,
            userData: {
                name: "Label_" + labelText
            }
        });
    }

});