﻿var KeyboardCopyPastePolicy = draw2d.policy.canvas.DefaultKeyboardPolicy.extend({
    NAME: "KeyboardCopyPastePolicy",

    init: function () {
        this._super();

        this.copiedBlocks = null;
    },

    onKeyDown: function (canvas, keyCode, shiftKey, ctrlKey) {
        let currentSelection = canvas.getSelection();
        let selectedFigures = currentSelection.getAll().asArray().filter(f => f instanceof draw2d.shape.basic.Rectangle);

        if (selectedFigures !== null && selectedFigures.length > 0 && ctrlKey && keyCode === 67) {
            this.copiedBlocks = selectedFigures;
        }
        else if (ctrlKey && keyCode === 86 && this.copiedBlocks) {
            for (let copiedBlock of this.copiedBlocks) {
                let copiedBlockX = copiedBlock.x + copiedBlock.width;
                let copiedBlockY = copiedBlock.y + copiedBlock.height;

                let memento = copiedBlock.getPersistentAttributes();

                // clone the selection
                let newBlock = copiedBlock.clone();

                // fix port locators
                let portsData = newBlock.getPorts();

                for (let port of portsData.data) {
                    let mementoPort = memento.ports.find(f => f.name == port.name);

                    let locatorKeys = Object.keys(mementoPort.locatorAttr);

                    for (let k of locatorKeys) {
                        port.locator[k] = mementoPort.locatorAttr[k];
                    }

                    port.locator.relocate(0, port);
                }

                // fix child figure locators
                if (memento.childFigureLocators) {
                    let childFigures = newBlock.getChildren();

                    childFigures.each(function (index, element) {
                        if (element.userData) {
                            let figureLocator = memento.childFigureLocators[element.userData.name];

                            if (figureLocator) {
                                newBlock.remove(element);
                                newBlock.add(element, figureLocator);
                            }
                        }
                    });
                }

                // add the clone to the canvas and start dragging the clone
                canvas.add(newBlock, copiedBlockX, copiedBlockY);
            }
        }
        else {
            this._super(canvas, keyCode, shiftKey, ctrlKey);
        }
    }
});