﻿let HttpRequestBlockContentHandler = {

    previousBlockParameters: {},

    initializeHttpRequestModal: function () {

        this.initializeBlockFooter();

        HttpRequestBlockContentHandler.previousBlockParameters = getPreviousBlockParameters();

        let httpRequestPropertiesElement = document.getElementById("httpRequestProperties");

        while (modalBlockParametersElement.lastElementChild) {
            modalBlockParametersElement.lastElementChild.style.display = "none";
            modalParametersContainerElement.appendChild(modalBlockParametersElement.lastElementChild);
        }

        modalBlockParametersElement.appendChild(httpRequestPropertiesElement);
        httpRequestPropertiesElement.style.display = "block";

        if (selectedBlock.userData) {
            if (selectedBlock.userData.requestUri)
                document.getElementById("httpRequestUri").value = selectedBlock.userData.requestUri;
            else
                document.getElementById("httpRequestUri").value = "";

            if (selectedBlock.userData.requestMethod) {
                let httpRequestMethodInputElement = document.getElementById("httpRequestMethod");
                httpRequestMethodInputElement.value = selectedBlock.userData.requestMethod;
                this.onHttpRequestMethodChanged(httpRequestMethodInputElement);
            }

            if (selectedBlock.userData.requestTimeout)
                document.getElementById("httpRequestTimeout").value = selectedBlock.userData.requestTimeout;
            else
                document.getElementById("httpRequestTimeout").value = "";

            if (selectedBlock.userData.contentType)
                document.getElementById("httpRequestContentType").value = selectedBlock.userData.contentType;

            if (selectedBlock.userData.requestBody)
                document.getElementById("httpRequestBody").value = selectedBlock.userData.requestBody;
        }

        this.getHttpRequestParameters("httpRequestParameters");
    },

    getHttpRequestParameters: function (parametersTableId) {
        let parametersTable = document.getElementById(parametersTableId);

        parametersTable.innerHTML = "";

        if (selectedBlock.userData && selectedBlock.userData.requestParameters) {
            for (let element of selectedBlock.userData.requestParameters) {
                let inputRow = this.formParameterRow(element.Name, element.Source, element.Value, element.SourceBlockId, element.SourceBlockParameterName);

                parametersTable.append(inputRow);
            }
        }
    },

    initializeBlockFooter: function () {
        let saveBtn = document.createElement("button");
        saveBtn.className = "modal-blackButton";
        saveBtn.style.width = "50%";
        saveBtn.innerHTML = localizedSaveChangesString;

        saveBtn.onclick = function () {
            modal.style.display = "none";
            HttpRequestBlockContentHandler.saveHttpRequestProperties("httpRequestUri", "httpRequestMethod", "httpRequestTimeout", "httpRequestContentType", "httpRequestBody", "httpRequestParameters");

            validateProcessScheme(null);
        }

        while (modalFooter.firstChild) {
            modalFooter.removeChild(modalFooter.firstChild);
        }
        modalFooter.appendChild(saveBtn);
    },

    saveHttpRequestProperties: function (httpRequestUriElementId, httpRequestMethodElementId, httpRequestTimeoutElementId, httpRequestContentTypeElementId, httpRequestBodyElementId, requestParametersElementId) {
        let httpRequestUriElement = document.getElementById(httpRequestUriElementId);
        let httpRequestMethodElement = document.getElementById(httpRequestMethodElementId);
        let httpRequestTimeoutElement = document.getElementById(httpRequestTimeoutElementId);
        let httpRequestContentTypeElement = document.getElementById(httpRequestContentTypeElementId);
        let httpRequestBodyElement = document.getElementById(httpRequestBodyElementId);

        selectedBlock.userData.requestUri = httpRequestUriElement.value;
        selectedBlock.userData.requestMethod = httpRequestMethodElement.value;
        selectedBlock.userData.requestTimeout = httpRequestTimeoutElement.value;
        selectedBlock.userData.contentType = httpRequestContentTypeElement.value;
        selectedBlock.userData.requestBody = httpRequestBodyElement.value;

        selectedBlock.userData.blockName = blockNameInputElement.value;

        let requestParametersElement = document.getElementById(requestParametersElementId);

        let requestParameters = [];

        for (let parameterElement of requestParametersElement.children) {
            let name = parameterElement.querySelector('input[data-target="name"]').value;

            let sourceSelectValue = parameterElement.querySelector('select[data-target="source"]').value;

            if (sourceSelectValue === "Value") {
                let paramValue = parameterElement.querySelector('input[data-target="value"]').value;
                requestParameters.push(new HttpRequestParameterTarget(name, sourceSelectValue, paramValue, null, null));
            }
            else if (sourceSelectValue === "Block") {
                let previousBlockParameterSelectElement = parameterElement.querySelector('select[data-target="blockParameter"]');
                let selectedOptionElement = previousBlockParameterSelectElement.querySelector('option:checked');
                let paramBlockId = selectedOptionElement.parentElement.dataset.blockId;

                requestParameters.push(new HttpRequestParameterTarget(name, sourceSelectValue, null, paramBlockId, selectedOptionElement.dataset.name));
            }
        }

        selectedBlock.userData.requestParameters = requestParameters;

        setBlockNameLabel(selectedBlock, "blockName", blockNameInputElement.value);
    },

    formParameterRow: function (paramName, source, sourceValue, sourceId, previousBlockParameterName) {
        let inputRow = document.createElement("tr");

        let paramNameInput = document.createElement("input");
        paramNameInput.className = "parametersInput";
        paramNameInput.dataset.target = "name";
        paramNameInput.value = paramName;

        let paramNameCell = document.createElement("td");
        paramNameCell.appendChild(paramNameInput);

        inputRow.appendChild(paramNameCell);

        let sourceChoiceControl = document.createElement("select");
        sourceChoiceControl.dataset.target = "source";
        sourceChoiceControl.className = "parametersSelect";

        let valueOption = document.createElement("option");
        valueOption.value = "Value";
        valueOption.text = "Value";

        let blockOption = document.createElement("option");
        blockOption.value = "Block";
        blockOption.text = "Block";

        sourceChoiceControl.add(valueOption);
        sourceChoiceControl.add(blockOption);

        if (source)
            sourceChoiceControl.value = source;

        sourceChoiceControl.addEventListener("change", this.changeDataSourceSelectVisibility);

        let sourceChoiceControlCell = document.createElement("td");
        sourceChoiceControlCell.appendChild(sourceChoiceControl);

        inputRow.appendChild(sourceChoiceControlCell);

        let parameterSelectCell = document.createElement("td");

        let valueInput = document.createElement("input");
        valueInput.style.display = "none";
        valueInput.dataset.target = "value";
        valueInput.className = "parametersInput";

        if (sourceValue)
            valueInput.value = sourceValue;

        parameterSelectCell.appendChild(valueInput);

        let previousBlockParamsSelectList = formPreviousBlockParametersSelectList(this.previousBlockParameters);
        previousBlockParamsSelectList.style.display = "none";
        previousBlockParamsSelectList.dataset.target = "blockParameter";
        previousBlockParamsSelectList.className = "parametersSelect";

        if (source === "Block") {
            previousBlockParamsSelectList.style.display = "inline";

            if (previousBlockParameterName) {
                previousBlockParamsSelectList.value = sourceId + "_" + previousBlockParameterName;
            }
        }
        else
            valueInput.style.display = "inline";

        parameterSelectCell.appendChild(previousBlockParamsSelectList);

        inputRow.appendChild(parameterSelectCell);

        let removeRowButton = document.createElement("input");
        removeRowButton.type = "button";
        removeRowButton.className = "modal-transparentButton";
        removeRowButton.value = "X";
        removeRowButton.addEventListener("click", this.removeRow);

        let removeRowButtonCell = document.createElement("td");
        removeRowButtonCell.appendChild(removeRowButton);

        inputRow.appendChild(removeRowButtonCell);

        return inputRow;
    },

    changeDataSourceSelectVisibility: function (e) {
        let parentRow = e.target.closest("tr");

        let valueSelectElement = parentRow.querySelector('input[data-target="value"]');
        let prevBlockParamSelectElement = parentRow.querySelector('select[data-target="blockParameter"]');

        if (e.target.value === "Value") {
            valueSelectElement.style.display = "inline";
            prevBlockParamSelectElement.style.display = "none";
        }
        else {
            valueSelectElement.style.display = "none";
            prevBlockParamSelectElement.style.display = "inline";
        }
    },

    addParameterRow: function (requestParametersDivId) {
        let requestParametersDiv = document.getElementById(requestParametersDivId);

        requestParametersDiv.appendChild(this.formParameterRow(null, null, null, null, null));
    },

    removeRow: function (e) {
        let parameterRow = e.target.closest("tr");
        parameterRow.remove();
    },

    onHttpRequestMethodChanged: function (inputElement) {

        let httpRequestPropertiesElement = document.getElementById("httpRequestProperties");
        let requestBodyRows = httpRequestPropertiesElement.querySelectorAll("[data-request-body]");

        if (inputElement.value === "Post" || inputElement.value === "Put")
            requestBodyRows.forEach(f => f.style.display = "table-row");
        else
            requestBodyRows.forEach(f => f.style.display = "none");
    }
}