﻿let TaskBlockContentHandler = {

    dataSelectIdPrefix: "dataSelect",

    blockParameterSelectIdPrefix: "blockParameterSelect",

    taskSelectDdlId: "taskSelect",

    previousBlockParameters: {},

    formParameterRow: function (paramId, paramType, paramName, source, sourceId, previousBlockParameterName) {
        let inputRow = document.createElement("tr");
        inputRow.id = paramId;
        inputRow.dataset["type"] = paramType;

        let paramNameLabel = document.createElement("label");
        paramNameLabel.textContent = paramName;

        let labelCell = document.createElement("td");
        labelCell.appendChild(paramNameLabel);

        inputRow.appendChild(labelCell);

        let sourceChoiceControl = document.createElement("select");
        sourceChoiceControl.id = "sourceChoice_" + paramId;
        sourceChoiceControl.className = "parametersSelect";
        sourceChoiceControl.style.minWidth = "70px";
        sourceChoiceControl.style.maxWidth = "70px";

        let dataOption = document.createElement("option");
        dataOption.value = "Data";
        dataOption.text = "Data";

        let blockOption = document.createElement("option");
        blockOption.value = "Block";
        blockOption.text = "Block";

        sourceChoiceControl.add(dataOption);
        sourceChoiceControl.add(blockOption);

        if (source)
            sourceChoiceControl.value = source;

        sourceChoiceControl.addEventListener("change", this.changeDataSourceSelectVisibility);

        let sourceChoiceControlCell = document.createElement("td");
        sourceChoiceControlCell.appendChild(sourceChoiceControl);

        inputRow.appendChild(sourceChoiceControlCell);

        let parameterSelectCell = document.createElement("td");
        parameterSelectCell.style.width = "100%";

        let dataSelect = document.getElementById("dataSelectViewBag").cloneNode(true);
        dataSelect.style.display = "none";
        dataSelect.id = this.dataSelectIdPrefix + "_" + paramId;
        dataSelect.className = "parametersSelect";

        let dataSelectOptions = dataSelect.options;

        for (let i = 0; i < dataSelectOptions.length; i++) {
            dataSelectOptions[i].dataValue = dataSelectOptions[i].text;
        }

        //let dataSelectObserver = new ResizeObserver(this.fitDataSelectOptionsWidth);
        //dataSelectObserver.observe(dataSelect);

        if (source === "Data") {
            dataSelect.value = sourceId;
            dataSelect.style.display = "inline";
        }

        parameterSelectCell.appendChild(dataSelect);

        let previousBlockParamsSelectList = formPreviousBlockParametersSelectList(this.previousBlockParameters);
        previousBlockParamsSelectList.style.display = "none";
        previousBlockParamsSelectList.id = this.blockParameterSelectIdPrefix + "_" + paramId;

        if (source === "Block") {
            previousBlockParamsSelectList.style.display = "inline";

            if (previousBlockParameterName) {
                previousBlockParamsSelectList.value = sourceId + "_" + previousBlockParameterName;
            }
        }

        parameterSelectCell.appendChild(previousBlockParamsSelectList);

        inputRow.appendChild(parameterSelectCell);

        return inputRow;
    },

    fitDataSelectOptionsWidth: function (dataSelectElementObservableEntry) {

        let dataSelectElement = dataSelectElementObservableEntry[0].target;

        let dataSelectOptions = dataSelectElement.options;

        for (let i = 0; i < dataSelectOptions.length; i++) {

            let option = dataSelectOptions[i];

            // регулируем скорость изменения длины строки одновременно с шириной выпадающего списка
            let trimMultiplier = 4.2;

            //// "-15" - задаем первоначальное выравнивание строки с шириной выпадающего списка
            //let lengthToShortenTo = Math.round(parseInt(option.parentElement.offsetWidth, 10) / trimMultiplier) - 15;

            let lengthToShortenTo = 20;

            if (option.dataValue.length > lengthToShortenTo) {
                option.text = option.dataValue.substring(0, lengthToShortenTo) + '...';
            }
        }
    },

    changeDataSourceSelectVisibility: function (e) {
        let paramId = e.target.id.split("_")[1];
        let parentDiv = e.target.closest("table");

        let dataSelectElement = parentDiv.querySelector("#" + TaskBlockContentHandler.dataSelectIdPrefix + "_" + paramId);
        let prevBlockParamSelectElement = parentDiv.querySelector("#" + TaskBlockContentHandler.blockParameterSelectIdPrefix + "_" + paramId);

        if (e.target.value === "Data") {
            dataSelectElement.style.display = "inline";
            prevBlockParamSelectElement.style.display = "none";
        }
        else {
            dataSelectElement.style.display = "none";
            prevBlockParamSelectElement.style.display = "inline";
        }
    },

    initializeTaskModal: function () {

        this.initializeBlockFooter();

        let taskSelectDdl = $("#" + this.taskSelectDdlId);

        if (selectedBlock && selectedBlock.userData && selectedBlock.userData.taskId)
            taskSelectDdl.val(selectedBlock.userData.taskId);

        this.getTaskParameters();

        let taskPropertiesElement = document.getElementById("taskProperties");

        while (modalBlockParametersElement.lastElementChild) {
            modalBlockParametersElement.lastElementChild.style.display = "none";
            modalParametersContainerElement.appendChild(modalBlockParametersElement.lastElementChild);
        }

        modalBlockParametersElement.appendChild(taskPropertiesElement);
        taskPropertiesElement.style.display = "block";
    },

    initializeBlockFooter: function () {
        let saveBtn = document.createElement("button");
        saveBtn.className = "modal-blackButton";
        saveBtn.style.width = "50%";
        saveBtn.innerHTML = localizedSaveChangesString;

        saveBtn.onclick = function () {
            modal.style.display = "none";
            TaskBlockContentHandler.saveTaskProperties(TaskBlockContentHandler.taskSelectDdlId, "taskParameters");

            validateProcessScheme(null);
        };

        let clearButton = document.createElement("button");
        clearButton.className = "modal-whiteButton";
        clearButton.style.width = "50%";
        clearButton.innerHTML = localizedSetParametersToDefaultString;

        clearButton.onclick = function () {
            TaskBlockContentHandler.clearParameters();
        };

        while (modalFooter.firstChild) {
            modalFooter.removeChild(modalFooter.firstChild);
        }

        modalFooter.appendChild(saveBtn);
        modalFooter.appendChild(clearButton);
    },

    getTaskParameters: function () {

        let selectedTaskId = $("#taskSelect :selected").val();

        let taskParametersTable = $("#taskParameters");
        taskParametersTable.empty();

        TaskBlockContentHandler.previousBlockParameters = getPreviousBlockParameters();

        if (selectedBlock.userData && selectedBlock.userData.parameters && selectedBlock.userData.taskId === selectedTaskId) {
            //Загрузка данных из сохранённых параметров блока
            for (let element of selectedBlock.userData.parameters) {
                let inputRow = TaskBlockContentHandler.formParameterRow(element.ParameterId, element.Type, element.Name, element.Source, element.SourceId, element.BlockParameterName);

                taskParametersTable.append(inputRow);
            }
        }
        else {
            if (selectedTaskId) {
                //Запрос данных параметров блока
                $.ajax({
                    url: "/BPM/GetTaskParameters",
                    type: "GET",
                    data: {
                        taskId: selectedTaskId
                    }
                }).done(function (response) {
                    for (let element of response) {
                        let inputRow = TaskBlockContentHandler.formParameterRow(element.paramId, element.paramType, element.paramName, "Data", element.dataId, null);

                        taskParametersTable.append(inputRow);
                    }
                });
            }
        }
    },

    // Сохранение свойств блока задачи в объекте draw2d
    saveTaskProperties: function (taskSelectElementId, taskParametersElementId) {

        let sel = document.getElementById(taskSelectElementId);
        let taskName = sel.options[sel.selectedIndex].text;
        let taskId = sel.options[sel.selectedIndex].value;

        this.setTaskToBlock(selectedBlock, taskName, taskId);

        let taskParametersElement = document.getElementById(taskParametersElementId);

        this.setParametersToTaskBlock(selectedBlock, taskParametersElement);
        selectedBlock.userData.blockName = blockNameInputElement.value;
    },

    setTaskToBlock: function (figure, taskName, taskId) {

        if (blockNameInputElement.value && blockNameInputElement.value.length > 0)
            setBlockNameLabel(figure, "taskName", blockNameInputElement.value);
        else
            setBlockNameLabel(figure, "taskName", taskName);

        figure.userData.taskId = taskId;
    },

    //Установка параметров для блока задачи
    /**
     * @method set an array of task parameters to target task block figure
     * @param {any} [figure] draw2d task block
     * @param {HTMLElement} [taskParametersElement] DOM element that contains task parameters
     */
    setParametersToTaskBlock: function (figure, taskParametersElement) {
        let taskParameters = [];

        for (let paramElement of taskParametersElement.children) {
            let id = paramElement.id;
            let type = paramElement.dataset["type"];

            let name = paramElement.getElementsByTagName("label")[0].textContent;

            let sourceSelectValue = $("#sourceChoice_" + id + " :selected").val();

            if (sourceSelectValue === "Data") {
                let dataSelectId = $("#dataSelect_" + id + " :selected").val();
                taskParameters.push(new TaskParameterTarget(name, type, sourceSelectValue, dataSelectId, null, id));
            }
            else if (sourceSelectValue === "Block") {
                let previousBlockParameter = $("#blockParameterSelect_" + id + " :selected");
                let previousBlockId = previousBlockParameter.parent().attr("data-block-id");

                taskParameters.push(new TaskParameterTarget(name, type, sourceSelectValue, previousBlockId, previousBlockParameter.attr("data-name"), id));
            }
        }

        figure.userData.parameters = taskParameters;
    },

    clearParameters: function () {
        selectedBlock.userData.parameters = null;
        this.getTaskParameters();
    }
}