﻿const draw2dGraphHelper = {
    getShortestPath : function(canvas, sourceBlockId, targetBlockId) {
        let adjacencyList = this.getAdjacencyList(canvas);

        let queue = [sourceBlockId];
        let visited = {};
        visited[sourceBlockId] = true;

        let predecessor = {};

        let path = [];
        let currentVertex;

        while (queue.length) {
            currentVertex = queue.shift();
            let adjacentVertices = adjacencyList[currentVertex];

            if (!adjacentVertices)
                continue;

            for (let v of adjacentVertices) {
                if (visited[v]) {
                    continue;
                }
                visited[v] = true;

                if (v === targetBlockId) {   // Check if the path is complete.
                    path = [v];   // If so, backtrack through the path.
                    while (currentVertex !== sourceBlockId) {
                        path.push(currentVertex);
                        currentVertex = predecessor[currentVertex];
                    }
                    path.push(currentVertex);
                    path.reverse();

                    break;
                }

                predecessor[v] = currentVertex;
                queue.push(v);
            }
        }

        return path;
    },

    getAdjacencyList: function (canvas) {

        let connections = [];
        canvas.getLines().each(function (i, element) {
            connections.push(element);
        });;

        let adjacencyList = {};

        for (let connection of connections) {
            let sourceBlockId = connection.getSource().getParent().getId();
            let targetBlockId = connection.getTarget().getParent().getId();

            if (!adjacencyList[sourceBlockId]) {
                adjacencyList[sourceBlockId] = [];
            }

            adjacencyList[sourceBlockId].push(targetBlockId);
        }

        return adjacencyList;
    }
}