﻿

function inputFile_Changed(inputId, selectId, labelId, dateInputId) {

    let lastEdit = document.getElementById(dateInputId);
    let input = document.getElementById(inputId);
    let select = document.getElementById(selectId);
    // Очистка выпадающего списка
    select.options.length = 0;

    let label = document.getElementById(labelId);

    var zipFile = input.files[0];

    let lastModifiedDate;

    if (zipFile.lastModifiedDate)
        lastModifiedDate = zipFile.lastModifiedDate;
    else
        lastModifiedDate = new Date(zipFile.lastModified);

    let date = formatDate(lastModifiedDate);
    let fileInfo = "Name: " + zipFile.name + " Size: " + zipFile.size + "b Last update: " + date;

    label.innerText = fileInfo;
    lastEdit.value =  lastModifiedDate.toISOString();

    var fileReader = new FileReader();

    fileReader.onload = function (fileLoadedEvent) {
        file_OnLoad(fileLoadedEvent, select);
    };

    fileReader.readAsArrayBuffer(zipFile);
}

const b64toBlob = (b64Data, contentType = '', sliceSize = 512) => {
    const byteCharacters = window.atob(b64Data);
    const byteArrays = [];

    for (let offset = 0; offset < byteCharacters.length; offset += sliceSize) {
        const slice = byteCharacters.slice(offset, offset + sliceSize);

        const byteNumbers = new Array(slice.length);
        for (let i = 0; i < slice.length; i++) {
            byteNumbers[i] = slice.charCodeAt(i);
        }

        const byteArray = new Uint8Array(byteNumbers);
        byteArrays.push(byteArray);
    }

    const blob = new Blob(byteArrays, { type: contentType });
    return blob;
}

function loadFile(zipFile, selectId, labelId, lastEditDate, entryPoint) {
     
    let select = document.getElementById(selectId);
    select.options.length = 0;
    let label = document.getElementById(labelId);
      
    let date = formatDate(new Date(Date.parse(lastEditDate)));
    let fileInfo =  zipFile.size + "kb Last update: " + date;
    
    label.innerText = fileInfo;

	var fileReader = new FileReader();

    fileReader.onload = function (fileLoadedEvent) {
        file_OnLoad(fileLoadedEvent, select);
    };

    select.value = entryPoint;
    fileReader.readAsArrayBuffer(zipFile);
}

function file_OnLoad (fileLoadedEvent, select) {
    var zipFileLoaded = new JSZip();

    var decoder = new TextDecoder('ibm866');
    zipFileLoaded.loadAsync(fileLoadedEvent.target.result, {
        decodeFileName: function (bytes) {
            return decoder.decode(bytes);
        }
    }).then(function (zip) {

        let fileEntries = Object.values(zip.files); 
        let projectFiles = fileEntries.filter(f => f.name.split('.').pop() === 'pixproj');

        if (projectFiles.length > 0) {
            for (let projectFile of projectFiles) {
                projectFile.async('string').then(function (fileData) {

                    let projectFilesRootElement = $(fileData).find("Files");

                    for (let fileItem of projectFilesRootElement[0].children) {

                        let isFileMain = fileItem.getElementsByTagName("ismain")[0].textContent;

                        if (isFileMain === "true") {

                            let filePath = fileItem.getElementsByTagName("path")[0].textContent;

                            let projectFileLastDashIndex = projectFile.name.lastIndexOf("/");

                            if (projectFileLastDashIndex > -1) {
                                let projectFilePath = projectFile.name.substring(0, projectFileLastDashIndex + 1);
                                filePath = projectFilePath + filePath;
                            }

                            select.options.add(new Option(filePath + " (" + projectFile.name + ")", filePath));
                        }
                    }
                });
            }
        }
        else {
            for (var fileName in zip.files) {
                let lastCharOfPath = fileName[fileName.length - 1];

                if (extname(fileName) != "" && lastCharOfPath != '/' && lastCharOfPath != '\\')
                    fillSelect(select, fileName);
            }
        }
    });
}

function formatDate(date) {
    console.log(date);
    let month = (date.getMonth() + 1) < 10 ? '0' + (date.getMonth() + 1) : (date.getMonth() + 1);
    let hours = date.getHours() < 10 ? '0' + date.getHours() : date.getHours();
    let minutes = date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes();
    let seconds = date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds();

    let dateStr = date.getDate() + "/" + month + "/" + date.getFullYear() + " " + hours + ":" + minutes + ":" + seconds;
    
    return dateStr;
}

function fillSelect(select, filePath) {  
    select.options.add(new Option(filePath));
}


var splitPathRe =
    /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;

function posixSplitPath(filename) {
    return splitPathRe.exec(filename).slice(1);
}

function extname(path) {
    return posixSplitPath(path)[3];
};

